;; -*-scheme-*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; advanced-portfolio-new.scm
;; by Thierry Scalais
;; modified for compatibility to GnuCash 2.2
;;     load definition file compat20.scm
;;     
;; modified from GnuCash 2.0.1
;; with patch from Scott Oonk to fix Basis truncation error (bug 343245)
;;
;; by Martijn van Oosterhout (kleptog@svana.org) Feb 2002
;; modified for GnuCash 1.8 by Herbert Thoma (herbie@hthoma.de) Oct 2002
;;
;; Heavily based on portfolio.scm
;; by Robert Merkel (rgmerk@mira.net)
;;
;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
;; Boston, MA  02110-1301,  USA       gnu@gnu.org
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-module (gnucash report advanced-portfolio-new))

(use-modules (gnucash main)) ;; FIXME: delete after we finish modularizing.
(use-modules (srfi srfi-1))
(use-modules (ice-9 slib))
(use-modules (gnucash gnc-module))
(require 'printf)

(gnc:module-load "gnucash/report/report-system" 0)

(load-from-path "gnucash/report/compat20.scm")
(define reportname (N_ "Advanced Portfolio New"))

(define optname-price-source (N_ "Price Source"))
(define optname-shares-digits (N_ "Share decimal places"))
(define optname-zero-shares (N_ "Include accounts with no shares"))
(define optname-include-gains (N_ "Include gains and losses"))
(define optname-show-symbol (N_ "Show ticker symbols"))
(define optname-show-listing (N_ "Show listings"))
(define optname-show-price (N_ "Show prices"))
(define optname-show-shares (N_ "Show number of shares"))
(define optname-basis-method (N_ "Basis calculation method"))
(define optname-prefer-pricelist (N_ "Set preference for price list data"))
(define optname-value-in-report-currency (N_ "Use the report currency to display the stock value"))
(define optname-show-return (N_ "Show the annual growth rate"))
(define optname-show-gains (N_ "Show gain in details"))

(define (options-generator)
  (let* ((options (gnc:new-options)) 
         ;; This is just a helper function for making options.
         ;; See gnucash/src/scm/options.scm for details.
         (add-option 
          (lambda (new-option)
            (gnc:register-option options new-option))))

    ;; General Tab
    ;; date at which to report balance
    (gnc:options-add-report-date!
     options gnc:pagename-general 
     (N_ "Date") "a")

    (gnc:options-add-currency! 
     options gnc:pagename-general (N_ "Report Currency") "c")

    (add-option
     (gnc:make-multichoice-option
      gnc:pagename-general optname-price-source
      "d" (N_ "The source of price information") 'pricedb-nearest
      (list (vector 'pricedb-latest 
		    (N_ "Most recent")
		    (N_ "The most recent recorded price"))
	    (vector 'pricedb-nearest
		    (N_ "Nearest in time")
		    (N_ "The price recorded nearest in time to the report date"))
	    (vector 'pricedb-latest-before
		    (N_ "Most recent to report")
		    (N_ "The most recent recorded price before report date"))
	    )))
    
    (add-option
     (gnc:make-multichoice-option
      gnc:pagename-general optname-basis-method
      "e" (N_ "Basis calculation method") 'average-basis
      (list (vector 'average-basis
		    (N_ "Average")
		    (N_ "Use average cost of all shares for basis"))
	    (vector 'fifo-basis
		    (N_ "FIFO")
		    (N_ "Use first-in first-out method for basis"))
	    (vector 'filo-basis
		    (N_ "FILO")
		    (N_ "Use first-in last-out method for basis"))
	    )))

    (add-option
     (gnc:make-simple-boolean-option
      gnc:pagename-general optname-prefer-pricelist "f" 
      (N_ "Prefer use of price editor pricing over transactions, where applicable.")
      #t))
    (gnc:register-option
      options
      (gnc:make-simple-boolean-option
        gnc:pagename-display optname-value-in-report-currency "a"
        (N_ "Use the report currency to display the stock value")
        #f))

    (gnc:register-option 
     options 
     (gnc:make-simple-boolean-option
      gnc:pagename-general optname-include-gains "g" 
      (N_ "Include splits with no shares for calculating gain")
      #f))

    (gnc:register-option
      options
      (gnc:make-simple-boolean-option
	gnc:pagename-display optname-show-symbol "a"
	(N_ "Display the ticker symbols")
	#t))

    (gnc:register-option
      options
      (gnc:make-simple-boolean-option
	gnc:pagename-display optname-show-listing "b"
	(N_ "Display exchange listings")
	#t))
  (gnc:register-option
      options
      (gnc:make-simple-boolean-option
  gnc:pagename-display optname-show-return "a"
  (N_ "Display annual growth rate")
  #t))
  (gnc:register-option
      options
      (gnc:make-simple-boolean-option
  gnc:pagename-display optname-show-gains "a"
  (N_ "Display gains in details")
  #t))
    (gnc:register-option
      options
      (gnc:make-simple-boolean-option
	gnc:pagename-display optname-show-shares "c"
	(N_ "Display numbers of shares in accounts")
	#t))

    (add-option
     (gnc:make-number-range-option
      gnc:pagename-display optname-shares-digits
      "d" (N_ "The number of decimal places to use for share numbers") 2
      0 6 0 1))

    (gnc:register-option
      options
      (gnc:make-simple-boolean-option
	gnc:pagename-display optname-show-price "e"
	(N_ "Display share prices")
	#t))

    ;; Account tab
    (add-option
     (gnc:make-account-list-option
      gnc:pagename-accounts (N_ "Accounts")
      "b"
      (N_ "Stock Accounts to report on")
;;2.0      (lambda () (filter gnc:account-is-stock?
;;2.0                         (gnc:group-get-subaccounts
;;2.0                          (gnc:get-current-group))))
      (lambda () (filter gnc:account-is-stock?
                   (gnc-account-get-descendants-sorted
                      (gnc-get-current-root-account))))
      (lambda (accounts) (list  #t 
                                (filter gnc:account-is-stock? accounts)))
      #t))

    (gnc:register-option 
     options 
     (gnc:make-simple-boolean-option
      gnc:pagename-accounts optname-zero-shares "e" 
      (N_ "Include accounts that have a zero share balances.")
      #f))
    
    (gnc:options-set-default-section options gnc:pagename-general)      
    options))

;; This is the rendering function. It accepts a database of options
;; and generates an object of type <html-document>.  See the file
;; report-html.txt for documentation; the file report-html.scm
;; includes all the relevant Scheme code. The option database passed
;; to the function is one created by the options-generator function
;; defined above.

(define (advanced-portfolio-renderer report-obj)
  
 (let ((work-done 0)
       (work-to-do 0)
       (warn-price-dirty #f))
;; this function computes the interest rate that once applied
;; on each transaction amount consisting to buy or sell shares
;; gives the current gain
;; Args:
;;     transfers is a list of couple constitued of a transaction
;;               amount and its posted date
;;     gain is current gain on these transactions. It contains
;;          incomes if 'include-gains' is true.
  (define (annualreturn transfers gain to-date)
    (let
      (
        (rate 1)
        (years 0.0)   ;; approx. mean duration of investment
        (buyval 0.0)  ;; total amount of buy
        (ratio 0.0)
        (totval 0.0)  ;; total amount of transactions
        (rateres 0.0) ;; computed return at the interest rate up to now
        (rateresderiv 0.0) ;; derivative of rateres with respect to the rate.
        (prevrate -1000000000.0) ;; rate computed at the precedent iteration
          (g (gnc:numeric-to-double (gnc:gnc-monetary-amount gain))) ;; value of gain
      )
      ;; function to totalise the amounts of acquisitions
      (define (buyvalue transfers to-date)
        (let
          ( (sum (gnc:numeric-zero)))
          ;(gnc:debug "annualreturn: buyvalue"  transfers)
          (if (eqv? transfers '()
              ) ;;!! ajouter condition sur la date
            (set! sum (gnc:numeric-zero))
            (let
              ( (v (if(gnc:numeric-positive-p (caar transfers))
                      (caar transfers)
                      (gnc:numeric-zero)
                    )))
              (set! sum (gnc:numeric-add
                          v
                          (buyvalue (cdr transfers) to-date)
                          100
                          GNC-RND-ROUND)))
          )
          ;(gnc:debug "annualreturn: buyvalue"  sum)
          sum)
      )
      ;; function to totalise the amounts of transactions
      (define (total transfers to-date)
        (let
          ( (sum (gnc:numeric-zero)))
         ; (gnc:debug "annualreturn: totalvalue"  transfers)
          (if (eqv? transfers '() ) ;;!! ajouter condition sur la date
            (set! sum (gnc:numeric-zero))
            (set! sum (gnc:numeric-add
                        (caar transfers)
                          (total (cdr transfers) to-date)
                        100
                        GNC-RND-ROUND))
          )
         ; (gnc:debug "annualreturn: totvalue"  sum)
          sum)
      )
      ;; function computing the summation of
      ;; the product of the transaction amount and the age of the transaction in years 
      (define (valueyears transfers gain to-date)
        (let
          ((sm 0)
           (t 0))
          (for-each
          (lambda(s)
            (let
              ( ( v (gnc:numeric-to-double (car s))))
              (set! t (gnc:date-year-delta (caar(cdr s)) (car to-date)))
              ;( gnc:debug "ratereturn" rate " - "v " - " t " - "( * v ( expt rate t)) " - " sm)
              (set! sm (+  sm ( * v t) ))
            )
          )
          transfers
          )
          (set!  sm (+ sm ( * (gnc:numeric-to-double (gnc:gnc-monetary-amount gain))  t)))
          sm
        )
      )
      ;; Function computing the return of the transactions amounts
      ;; at the annual growth rate 'rate'
      ;; That is the summation of (amount x (rate^duration))
       
      (define (ratereturn rate transfers to-date)
        (let
          ((sm 0))
          (for-each
          (lambda(s)
            (let
              ( ( v (gnc:numeric-to-double (car s)))
                ( t (gnc:date-year-delta (caar(cdr s)) (car to-date))))
              ;( gnc:debug "ratereturn" rate " - "v " - " t " - "( * v ( expt rate t)) " - " sm)
              (set! sm (+  sm ( * v ( expt rate t)) ))
            )
          )
          transfers
          )
          sm
        )
      )
      ;; Function computing the derivative with respect of the rate
      (define (ratereturn-derivative rate transfers to-date)
        (let
          ((sm 0))
          (for-each
          (lambda(s)
            (let
              ( ( v (gnc:numeric-to-double (car s)))
                ( t (gnc:date-year-delta (caar(cdr s)) (car to-date))))
              (set! sm (+  sm (* ( * v ( expt  rate (- t 1)))  t) ))
              ;( gnc:debug "ratereturn-derivative" rate " - "v " - " t " - "( * v ( expt rate ( - t 1))) " - " sm)
            )
          )
          transfers
          )
          sm
        ))
      (set! buyval  (gnc:numeric-to-double (buyvalue transfers to-date)))
      (set! totval  ( + g (gnc:numeric-to-double (total transfers to-date))))

      (cond( ( and (not ( = 0 g)) (>  buyval 0))
        (set! years   (/  (valueyears transfers gain to-date) buyval))
        (set! ratio   (/ g buyval )) ; buyval must be non null
        (set! rate    (expt ( + ratio 1.0) (/ 1.0 years))) ; ratio must be >-1, years must be >0
        ;; this a first approximation. We will enhance tha accuracy of tis appoximation
        (set! rateres (ratereturn rate transfers to-date))
        (set! rateresderiv (ratereturn-derivative rate transfers to-date))
        ;; acuracy enhancement
        ;; the returnrate is computed by a iterative process upto an accuracy of 0.01%
        ;; we need a iterative process to increase the accuracy
        ;; this uses the Newton Raphson method
        ;(gnc:debug "annualreturn: buyvalue result" buyval)
        ;(gnc:debug "annualreturn: gain " g)
        ;(gnc:debug "annualreturn: years result" years)
        ;(gnc:debug "annualreturn: ratio " ratio)
        ;(gnc:debug "annualreturn: total "  totval )
        ;(gnc:debug "annualreturn: rate before " rate)
        ;(gnc:debug "annualreturn: rate total " rateres  )
        ;(gnc:debug "annualreturn: rate total error" (abs (- rateres totval))  )
        ;(gnc:debug "annualreturn: ratederiv"  rateresderiv)
(cond ((and (> years 0))
        (do
          (( i 0 (+ 1 i)))
          ((or (> i 20) (< (abs (- rate prevrate)) 0.001) ))
          (set! prevrate rate)
    ;      (set! rate (+ rate (-(/ (log totval) (log rateres)) 1 )))
          (set! rate (- rate ( / (- rateres totval) rateresderiv ))) ;; rateresderiv shouldn't be null
          (set! rateres (ratereturn rate transfers to-date))
          (set! rateresderiv (ratereturn-derivative rate transfers to-date))

        ;(gnc:debug "iterations" i " rate error"  (abs (- rate prevrate)))
        ;(gnc:debug "annualreturn: rate AFTER" rate)
        ;(gnc:debug "annualreturn: rate total " rateres  )
        ;(gnc:debug "annualreturn: rate total error" (abs (- rateres totval))  )
        ;(gnc:debug "annualreturn: ratederiv"  rateresderiv)
)
        )(else (set! rate 0)))
        ))
    ( -  rate 1)))
;;  (define (monetary-zero currency)
;;    (gnc:make-gnc-monetary  currency))

  (define (monetary-add a b denom how)
    (if(gnc:commodity-equiv? (gnc:gnc-monetary-commodity a) (gnc:gnc-monetary-commodity b))
      #t
      (gnc:warn "monetary-add: commodity of" (gnc:monetary->string a)
                "\nis different from "(gnc:monetary->string b)
                "\n->wrong addition result")
               )
    (gnc:make-gnc-monetary  (gnc:gnc-monetary-commodity a)
                            (gnc:numeric-add
                              (gnc:gnc-monetary-amount a)
                              (gnc:gnc-monetary-amount b)
                              denom
                              how
                            )))

  (define (monetary-sub a b denom how)
    ;(gnc:debug "monetary-sub:" (gnc:monetary->string a) " "(gnc:monetary->string b)" " denom " " how)
    (monetary-add a (gnc:monetary-neg b) denom how ))
                
  ;; These are some helper functions for looking up option values.
  (define (get-op section name)
    (gnc:lookup-option (gnc:report-options report-obj) section name))
  
  (define (get-option section name)
    (gnc:option-value (get-op section name)))
  
;;2.0  (define (split-account-type? split type)
;;2.0    (eq? type 
;;2.0         (gw:enum-<gnc:AccountType>-val->sym (gnc:account-get-type (gnc:split-get-account split)) #f)))
;;2.0  (define (split-get-account-type split )
;;2.0         (gw:enum-<gnc:AccountType>-val->sym (gnc:account-get-type (gnc:split-get-account split)) #f))
    (define (split-account-type? split type)
      (eq? type (xaccAccountGetType (xaccSplitGetAccount split))) ;;advanced-portfolio 2.2
    )
    (define (split-get-account-type split )
      ( xaccAccountGetType (xaccSplitGetAccount split))
    )
  ;;compare identity between two split based on their UID
  (define (same-split? s1 s2)
    (string=? (gnc:split-get-guid s1) (gnc:split-get-guid s2)))
  ;;compare identity between two split based on their UID
  (define (same-account? a1 a2)
    (string=? (gnc:account-get-guid a1) (gnc:account-get-guid a2)))
  
  ;; this builds a list for basis calculation and handles average, fifo and filo methods
  ;; the list is cons cells of (units-of-stock . price-per-unit)... average method produces only one
  ;; cell that mutates to the new average. Need to add a date checker so that we allow for prices
  ;; coming in out of order, such as a transfer with a price adjusted to carryover the basis.
  (define (basis-builder b-list b-units b-value b-method)
    ;(gnc:debug "basis builder" (gnc:numeric-to-double b-units)) 
    (if (gnc:numeric-positive-p b-units)

      (case b-method
      ;;Average method
        ( (average-basis.)
          (if (not (eqv? b-list '()))
            (list (cons (gnc:numeric-add  b-units
                                          (caar b-list)
                                          GNC-DENOM-AUTO
                                          (logior GNC-DENOM-REDUCE GNC-RND-NEVER))
                        (gnc:numeric-div
                          (gnc:numeric-add b-value
                                          (gnc:numeric-mul (caar b-list)
                                                            (cdar b-list)
                                                            GNC-DENOM-AUTO
                                                            (logior
                                                            GNC-DENOM-REDUCE
                                                            GNC-RND-NEVER))
                                          GNC-DENOM-AUTO
                                          (logior GNC-DENOM-REDUCE GNC-RND-NEVER))
                          (gnc:numeric-add b-units
                                          (caar b-list)
                                          GNC-DENOM-AUTO
                                          (logior GNC-DENOM-REDUCE GNC-RND-NEVER))
                          GNC-DENOM-AUTO
                          (logior GNC-DENOM-REDUCE GNC-RND-NEVER)
                        )
                  )
            )
            (append b-list
                    (list (cons b-units (gnc:numeric-div
                                          b-value b-units GNC-DENOM-AUTO
                                          (logior GNC-DENOM-REDUCE GNC-RND-NEVER))))
            )
          )
        )
        (else (append b-list
                            (list (cons b-units (gnc:numeric-div 
                                                b-value b-units GNC-DENOM-AUTO 
                                                (logior GNC-DENOM-REDUCE GNC-RND-NEVER))
                                  )
                            )
              )
        )
      )

      (if (not (eqv? b-list '()))
        (case b-method
          ;;FIFO method
          ((fifo-basis)
            (if  (not (= -1 (gnc:numeric-compare
                            (gnc:numeric-abs b-units) (caar b-list)))
                  )
                  (basis-builder (cdr b-list) (gnc:numeric-add
                                                b-units
                                                (caar b-list)
                                                GNC-DENOM-AUTO
                                                (logior GNC-DENOM-REDUCE GNC-RND-NEVER)
                                              )
                                b-value b-method
                  )
                  (append (list (cons (gnc:numeric-add
                                        b-units
                                        (caar b-list)
                                        GNC-DENOM-AUTO
                                        (logior GNC-DENOM-REDUCE GNC-RND-NEVER)
                                      )
                                    (cdar b-list)
                                )
                          )
                        (cdr b-list)
                  )
              )
          )
          ;;FILO method
          ( (filo-basis)
            (if (not (= -1 (gnc:numeric-compare
                            (gnc:numeric-abs b-units) (caar (reverse b-list)))))
                (basis-builder (reverse (cdr (reverse b-list)))
                                (gnc:numeric-add
                                b-units
                                (caar (reverse b-list))
                                GNC-DENOM-AUTO (logior GNC-DENOM-REDUCE GNC-RND-NEVER))
                                b-value b-method)
                (append (cdr (reverse b-list))
                        (list (cons (gnc:numeric-add
                                      b-units
                                      (caar (reverse b-list))
                                      GNC-DENOM-AUTO
                                      (logior GNC-DENOM-REDUCE GNC-RND-NEVER))
                                    (cdar (reverse b-list)))))
            )
          )
          ;;Average method
          ( (average-basis)
            (list (cons (gnc:numeric-add
                            (caar b-list)
                            b-units
                            GNC-DENOM-AUTO
                            (logior GNC-DENOM-REDUCE GNC-RND-NEVER))
                        (cdar b-list)))
            )
        )
        '()
      )
    )
  );;define (basis-builder...

  ;; sum up the contents of the b-list built by basis-builder above
  (define (sum-basis b-list)
    (if (not (eqv? b-list '()))
        (gnc:numeric-add (gnc:numeric-mul (caar b-list) (cdar b-list) 100 GNC-RND-ROUND)
                         (sum-basis (cdr b-list)) 100 GNC-RND-ROUND)
        (gnc:numeric-zero)
    ))
  
  ;; sum up the total number of units in the b-list built by basis-builder above
  (define (units-basis b-list)
    (if (not (eqv? b-list '()))
        (gnc:numeric-add (caar b-list) (units-basis (cdr b-list)) 100 GNC-RND-ROUND)
        (gnc:numeric-zero)
    ))

(define (table-add-stock-rows table accounts to-date
                              currency price-fn exchange-fn 
                              include-empty include-gains show-symbol show-listing
                              show-shares show-price show-return show-gains
                              basis-method prefer-pricelist
                              total-basis total-value total-moneyin total-moneyout total-return
                              total-benefits total-gain total-ugain total-allgain
        )
  (let ((share-print-info
	        (gnc:share-print-info-places
	          (inexact->exact (get-option
                              gnc:pagename-display
                              optname-shares-digits
                            )
            )))
        (total-transfers '()))
;;  (define (totalize coll currency exchange-fn)
;;    (gnc:sum-collector-commodity coll currency exchange-fn)
;;  )
    
    (define (table-add-stock-rows-internal accounts odd-row?)
      (if (null? accounts) total-value
          (let* ( (row-style (if odd-row? "normal-row" "alternate-row"))
                  (current (car accounts))
                  (rest (cdr accounts))
                  (name (gnc:account-get-name current))
                  (commodity (gnc:account-get-commodity current))
                  (ticker-symbol (gnc:commodity-get-mnemonic commodity))
                  (listing (gnc:commodity-get-namespace commodity))
                  (unit-collector (gnc:account-get-comm-balance-at-date
                                    current
                                    to-date
                                    #f
                                  ))
                  (units (cadr (unit-collector 'getpair commodity #f)))
;; Counter to keep track of stuff
                  (unitscoll     (gnc:make-commodity-collector)) 
                  (moneyincoll   (gnc:make-commodity-collector)) ;; buy
                  (moneyoutcoll  (gnc:make-commodity-collector)) ;; sell
                  (benefits    (gnc:make-commodity-collector))
                  (gaincoll      (gnc:make-commodity-collector))
                  (price-list    (price-fn commodity to-date))
                  (price         (if (> (length price-list) 0) (car price-list) #f))
;; if there is no price, set a sane commod-currency for those zero-share
;; accounts. if its a no price account with shares, we'll get a currency later.
                  (commod-currency (if price (gnc:price-get-currency price) currency))
                  (value (exchange-fn (gnc:make-gnc-monetary commodity units) currency))
                  (txn-date to-date)
                  (pricing-txn #f)
                  (last-price (gnc:numeric-zero))
                  (use-txn #f)
                  (basis-list '())
                  (txn-currency currency)
                  (lastparent-initialised #f )
                  (lastparentguid "") ;; to track splits from the same transaction
                  (it-is-a-new-transaction #f)
                  (transfers '()) ;; couples of buy or sell amounts with their corresponding posted dates
                )


            (gnc:debug "---" name "---")
            (for-each
              (lambda (splits)



  ;;(gnc:debug "transaction account" (gnc:account-get-name(gnc:split-get-account splits)))
                (set! work-done (+ 1 work-done))
                (gnc:report-percent-done (* 100 (/ work-done work-to-do)))
                (let*  (
                        (parent (gnc:split-get-parent splits))
                        (parentguid (gnc:transaction-get-guid parent))
                        (txn-value (gnc:numeric-zero))
                        (txn-currency (gnc:transaction-get-currency (gnc:split-get-parent splits)))
                      )
  ;                    (gnc:debug
  ;                      " IN lastparent-initialised" lastparent-initialised
  ;                      "it-is-a-new-transaction" it-is-a-new-transaction
  ;                      "parent" parentguid
  ;                     "lastparent" lastparentguid
  ;             )
                  (cond ( lastparent-initialised
                          ( set! it-is-a-new-transaction (not (string=? parentguid lastparentguid)))
                        )
                        ( else

                          (set! it-is-a-new-transaction #t)
                          (set! lastparent-initialised #t)
                        )
                  )
                  (set! lastparentguid parentguid)
  ;;              (gnc:debug "OUT lastparent-initialised" lastparent-initialised
  ;;                         "it-is-a-new-transaction" it-is-a-new-transaction
  ;;                         "parent" parentguid
  ;;                         "lastparent" lastparentguid
  ;              )
                  (if
                    ( gnc:timepair-le (gnc:transaction-get-date-posted parent) to-date )
                    (begin
                      (for-each
                        (lambda (s)
                          (gnc:debug "1 -amount " (gnc:numeric-to-double (gnc:split-get-amount s))
                                    " acct " (gnc:account-get-name (gnc:split-get-account s))
                                    " type " (split-get-account-type s))
                          (gnc:debug "1 - value " (gnc:numeric-to-double (gnc:split-get-value s))
                                    " in " (gnc:commodity-get-printname commod-currency)
                                    " from " (gnc:transaction-get-description (gnc:split-get-parent s))
                                  )
        ;; If this is an asset type account for buy or sell, then grab a
        ;; currency and a txn-value for later computation
                        ;; (gnc:debug "  txn-units search" (gnc:account-get-name(gnc:split-get-account s)))
                          (cond
                            ((and (not (same-account? current (gnc:split-get-account s)))
                                  (not (or(split-account-type? s 'expense)
                                          (split-account-type? s 'income)
                                        )
                                  )
                              )

                              (if ;;only change the commod-currency if price failed
                                (not price)
                                (set! commod-currency (gnc:account-get-commodity (gnc:split-get-account s))))
                              (set! txn-value (gnc:numeric-abs (gnc:split-get-value s)))
                            )
                            ( (same-account? current (gnc:split-get-account s))
                              (set! last-price (gnc:split-get-share-price s))
                              (set! pricing-txn parent)
                              (set! txn-date (gnc:transaction-get-date-posted parent))
                            )
                          )
                        )
                        (gnc:transaction-get-splits parent)
                      )
                ;; go build the basis-list
                ;; the use of exchange-fn here is an attempt to get the basis list into one
                ;; currency to help accomodate stock transfers and other things. might not work.
                ;; amount of the split, not the amount of the transaction
                      (set! basis-list(basis-builder basis-list
                                        (gnc:split-get-amount splits)
                                        (gnc:gnc-monetary-amount
                                          (exchange-fn
                                            (gnc:make-gnc-monetary
                                              txn-currency
                                              txn-value
                                            )
                                            currency
                                          )
                                        )
                                        basis-method
                                      )
                      )
                      ;; collecting acquisitions, sells and the sum of expenses and incomes
                        (for-each
                        (lambda (s)(gnc:warn "-")
                          (gnc:warn "amount " (gnc:numeric-to-double (gnc:split-get-amount s))
                                      " acct " (gnc:account-get-name (gnc:split-get-account s))
                                      " type " (split-get-account-type s))
                          (gnc:warn "value " (gnc:numeric-to-double (gnc:split-get-value s))
                                      " in " (gnc:commodity-get-printname commod-currency)
                                      " from " (gnc:transaction-get-description (gnc:split-get-parent s))
                                      )
                          (cond
                            ( (same-split? s splits)
                              (gnc:warn "   Same splits")

                              (cond
                                ( (not (gnc:numeric-zero-p (gnc:split-get-amount s)))  ;; Is a shares transaction
                                  (unitscoll 'add commodity (gnc:split-get-amount s))
                                  (let
                                    ((elm (list (gnc:split-get-value s) (gnc:transaction-get-date-posted parent))))
                                    (if(eqv? transfers '())
                                      (set! transfers (list elm))
                                      (append! transfers (list elm))))
                                  (cond
                                    ( (gnc:numeric-negative-p (gnc:split-get-amount s)) ;; It is a sale
                                      (moneyoutcoll
                                        'add
                                        txn-currency
                                        (gnc:numeric-neg (gnc:split-get-value s))
                                      )
                                      (gnc:warn "    Sale"(gnc:numeric-to-double(gnc:split-get-value s)))
                                    )
                                    ( (gnc:numeric-positive-p (gnc:split-get-amount s)) ;; It is an acquisition
                                      (moneyincoll
                                        'add
                                        txn-currency
                                        (gnc:numeric-neg (gnc:split-get-value s))
                                      )
                                      (gnc:warn "    Acquisition"(gnc:numeric-to-double(gnc:split-get-value s)))
                                    )
                                  )
                                )
                                ( #t
                                  (cond
                                    ( (gnc:numeric-positive-p (gnc:split-get-value s))
                                      (gaincoll
                                        'add
                                        txn-currency
                                        ;;(gnc:numeric-neg (gnc:split-get-value s))
                                          (gnc:numeric-neg(gnc:split-get-value s))
                                      )
                                      (gnc:warn
                                        "    include gains >0"
                                      (gnc:monetary->string(gnc:make-gnc-monetary txn-currency (gnc:split-get-value s) ))
                                        "totalising "
                                        (gnc:monetary->string(gnc:sum-collector-commodity gaincoll currency exchange-fn)))
                                    )
                                    (else
                                      (gaincoll
                                        'add
                                        txn-currency
                                        (gnc:numeric-neg (gnc:split-get-value s))

                                      )
                                      (gnc:warn  "   include gains <0"
                                        (gnc:monetary->string(gnc:make-gnc-monetary txn-currency (gnc:split-get-value s)))
                                        "totalising "
                                        (gnc:monetary->string(gnc:sum-collector-commodity gaincoll currency exchange-fn))
                                      )
                                    )
                                  )

                                )
                              )
                            )
                            ( (and it-is-a-new-transaction
                                  (or(split-account-type? s 9);'expense)
                                    (split-account-type? s 8)));'income));;)

                              (gaincoll
                                'add
                                txn-currency
                                (gnc:numeric-neg(gnc:split-get-value s))
                              )
      ;;                         (gnc:warn "expense ->gaincoll"
      ;;                           (gnc:monetary->string(gnc:make-gnc-monetary txn-currency (gnc:split-get-value s)))
      ;;                           "totalising "
      ;;                           (gnc:monetary->string(gnc:sum-collector-commodity gaincoll currency exchange-fn))
      ;;                         )
                            )
                          )
                        );;for each s
                        (gnc:transaction-get-splits parent)
                      )
                    )
                  )
                )
              )
              (gnc:account-get-split-list current)
            );;for ech splits

;; now we determine which price data to use, the pricelist or the txn
;; and if we have a choice, use whichever is newest.
            (set! use-txn (if (not price) #t
                            (if prefer-pricelist #f
                              (if (not (gnc:timepair-le txn-date (gnc:price-get-time price))) #t
                                  #f
                              )
                            )
                          )
            )

	    ;; okay we're using the txn, so make a new price, value etc. and warn the user
          (if use-txn
            (begin
              (set! price (if (not (gnc:numeric-zero-p last-price))
                              (gnc:make-gnc-monetary commod-currency last-price)
                              (gnc:make-gnc-monetary commod-currency (gnc:numeric-zero))
                          ))
              (set! value (if price (gnc:make-gnc-monetary commod-currency
                                      (gnc:numeric-mul units
                                        (gnc:gnc-monetary-amount price)
                                         100
                                         GNC-RND-ROUND
                                      )
                                    )
                                    (gnc:make-gnc-monetary commod-currency (gnc:numeric-zero))
                          ))
              (set! warn-price-dirty #t)
            )
          )

;; what this means is gain = moneyout - moneyin + basis-of-current-shares, and
;; adjust for brokers and dividends.
          (gaincoll 'add currency (sum-basis basis-list))
          (gaincoll 'merge moneyoutcoll #f)
          (gaincoll 'merge moneyincoll #f)
;;  unrealised gain = Value - Base
;;  realised gain  = Expense - Income + ( Output + Base - Input)
;;  benefits = Base + MoneyIn - MoneyOut

          (if (or include-empty (not (gnc:numeric-zero-p units)))
            (let* ( (moneyin (gnc:monetary-neg  (gnc:sum-collector-commodity
                                                  moneyincoll
                                                  currency
                                                  exchange-fn
                                                )
                            ))
                    (moneyout (gnc:sum-collector-commodity moneyoutcoll currency exchange-fn))
;; just so you know, gain == realized gain, ugain == un-realized gain, bothgain, well..
                    (gain (gnc:sum-collector-commodity gaincoll currency exchange-fn))
                    (ugain (gnc:make-gnc-monetary currency
                              (gnc:numeric-sub
                                (gnc:gnc-monetary-amount (exchange-fn value currency))
                                (sum-basis basis-list) 
                                100
                                GNC-RND-ROUND
                              )
                            ))
                    (bothgain (gnc:make-gnc-monetary currency (gnc:numeric-add
                                                                (gnc:gnc-monetary-amount gain)
                                                                (gnc:gnc-monetary-amount ugain)
                                                                100
                                                                GNC-RND-ROUND
                                                              )
                              ))
                    (benefits (gnc:make-gnc-monetary currency
                                  (gnc:numeric-add(gnc:numeric-sub
                                                    (gnc:gnc-monetary-amount moneyout)
                                                    (gnc:gnc-monetary-amount moneyin)
                                                    100
                                                    GNC-RND-ROUND
                                                )
                                                (gnc:gnc-monetary-amount
                                                  (gnc:make-gnc-monetary currency (sum-basis basis-list)))
                                                100
                                                GNC-RND-ROUND
                                  )
                                 ))
                     (dividend (gnc:make-gnc-monetary currency
                                  (gnc:numeric-sub
                                      (gnc:gnc-monetary-amount gain)
                                      (gnc:gnc-monetary-amount benefits)
                                      100
                                      GNC-RND-ROUND
                                  )
                                ))
                    (activecols (list (gnc:html-account-anchor current)))
                    (returnrate 0)
                    
                  ) ;; let ...
                  ;(gnc:debug "Money in =" (gnc:monetary->string moneyin)
                  ;           "Money Out ="(gnc:monetary->string moneyout)
                  ;           "Gain ="(gnc:monetary->string gain)
                  ;           "UGain ="(gnc:monetary->string ugain)
                  ;           "Sell Gain="(gnc:monetary->string benefits)
                  ;           "Both Gain="(gnc:monetary->string bothgain)
                  ;           "Dividend="(gnc:monetary->string dividend)
                  ;           "Base="(gnc:numeric-to-double(sum-basis basis-list))
                  ;)
                  
                  (if include-gains #t
                    (begin
                      (set! bothgain (monetary-sub  bothgain dividend 100 GNC-RND-ROUND))
                      (set! dividend (gnc:make-gnc-monetary
                                      (gnc:gnc-monetary-commodity dividend)
                                      (gnc:numeric-zero))))
                  )
                  ;; let compute the annual growth rate
                  (if show-return (set! returnrate (annualreturn transfers bothgain to-date)))
                  
                  (if(eqv? total-transfers '())
                  (set! total-transfers  transfers)
                  (append! total-transfers  transfers))
                  
                  (total-value 'add (gnc:gnc-monetary-commodity value) (gnc:gnc-monetary-amount value))
                  (total-benefits 'add (gnc:gnc-monetary-commodity benefits) (gnc:gnc-monetary-amount benefits ))
                  ;;(gnc:debug "Total Plus Value" (gnc:numeric-to-double(gnc:gnc-monetary-amount (gnc:sum-collector-commodity total-benefits currency exchange-fn))))
                  (total-moneyin 'merge moneyincoll #f)
                  (total-moneyout 'merge moneyoutcoll #f)
                  (total-gain 'add (gnc:gnc-monetary-commodity dividend) (gnc:gnc-monetary-amount dividend))
                  ;(total-gain 'merge gaincoll #f)
                  (total-ugain 'add (gnc:gnc-monetary-commodity ugain) (gnc:gnc-monetary-amount ugain))
                  (total-allgain 'add (gnc:gnc-monetary-commodity bothgain) (gnc:gnc-monetary-amount bothgain))
                  (total-basis 'add currency (sum-basis basis-list))
;; build a list for the row  based on user selections
                  (if show-symbol (append! activecols (list ticker-symbol)))
                  (if show-listing (append! activecols (list listing)))
                  (if show-shares (append! activecols (list (gnc:make-html-table-header-cell/markup
                                                              "number-cell"
                                                               (gnc:amount->string units share-print-info)
                                                            )
                                                      )
                                  )
                  )
                  (if show-price
                    (append! activecols
                      (list (gnc:make-html-table-header-cell/markup
                              "number-cell"
                              (if use-txn
                                (gnc:html-transaction-anchor pricing-txn price)
                                (gnc:html-price-anchor price
                                  (let
                                    ((prc (gnc:make-gnc-monetary
                                                  (gnc:price-get-currency price)
                                                  (gnc:price-get-value price)
                                                )))
                                    (if (get-option gnc:pagename-display optname-value-in-report-currency)
                                        (set! prc (exchange-fn prc currency )))
                                    prc
                                  )
                                )
                              )
                            )
                      )
                    )
                  )
                  (append! activecols (list
                                        (if use-txn "*" " ")
                                        (gnc:make-html-table-header-cell/markup 
                                          "number-cell"
                                          ;(exchange-fn
                                            (gnc:make-gnc-monetary currency
                                              (gnc:numeric-div
                                                (sum-basis basis-list)
                                                units
                                                100
                                                GNC-RND-ROUND
                                              )
                                            )
                                        ;    commod-currency)
                                        )
                                        (gnc:make-html-table-header-cell/markup "number-cell" value)
                                        (gnc:make-html-table-header-cell/markup "number-cell" moneyin)
                                        (gnc:make-html-table-header-cell/markup "number-cell" moneyout)))
                  (if show-gains
                    (append! activecols (list (gnc:make-html-table-header-cell/markup "number-cell" benefits))))
                  (if (and show-gains include-gains)
                    (append! activecols (list  (gnc:make-html-table-header-cell/markup "number-cell" dividend))))
                  (if show-gains
                    (append! activecols (list
                                          (gnc:make-html-table-header-cell/markup "number-cell" ugain))))
                  (append! activecols (list
                                        (gnc:make-html-table-header-cell/markup "number-cell" bothgain)
                                        (gnc:make-html-table-header-cell/markup "number-cell"
                                          (let ((moneyinvalue (gnc:numeric-to-double (gnc:gnc-monetary-amount moneyin))))
                                              (if (= 0.0 moneyinvalue)
                                                  (sprintf #f "%.2f%%" moneyinvalue)
                                                  (sprintf #f "%.2f%%"
                                                          (* 100 (/ (gnc:numeric-to-double
                                                                      (gnc:gnc-monetary-amount bothgain))
                                                                      moneyinvalue)
                                                          )))))
                                      ))
                  (if show-return
                    (append! activecols (list
                                        (gnc:make-html-table-header-cell/markup "number-cell"
                                          (sprintf #f "%.2f%%" (* returnrate 100)))
                                      )))
                  (gnc:html-table-append-row/markup! table row-style activecols)
                  (table-add-stock-rows-internal rest (not odd-row?))
            )
            (table-add-stock-rows-internal rest odd-row?)
          )
          (gnc:price-list-destroy price-list)
        )
      );;if (null? accounts) ...

    );;define (table-add-stock-rows-internal accounts odd-row?) ...

    
    (set! work-to-do (gnc:accounts-count-splits accounts))
    (table-add-stock-rows-internal accounts #t)
    ( if show-return
      ( total-return 'add (annualreturn
        total-transfers
        (gnc:sum-collector-commodity total-allgain currency exchange-fn)
        to-date)))
    ;(gnc:debug total-transfers " return" (total-return 'total #f))
    )
  );; (define (table-add-stock-rows ...

;; Tell the user that we're starting.
  (gnc:report-starting reportname)

;; The first thing we do is make local variables for all the specific
;; options in the set of options given to the function. This set will
;; be generated by the options generator above.
  (let ((to-date      (gnc:date-option-absolute-time
                      (get-option gnc:pagename-general    "Date")))
        (accounts     (get-option gnc:pagename-accounts   "Accounts"))
        (currency     (get-option gnc:pagename-general    "Report Currency"))
        (price-source (get-option gnc:pagename-general     optname-price-source))
        (report-title (get-option gnc:pagename-general     gnc:optname-reportname))
        (include-empty (get-option gnc:pagename-accounts   optname-zero-shares))
        (include-gains (get-option gnc:pagename-general    optname-include-gains))
        (show-symbol  (get-option gnc:pagename-display     optname-show-symbol))
        (show-listing (get-option gnc:pagename-display     optname-show-listing))
        (show-shares  (get-option gnc:pagename-display     optname-show-shares))
        (show-price   (get-option gnc:pagename-display     optname-show-price))
        (show-return  (get-option gnc:pagename-display     optname-show-return))
        (show-gains  (get-option gnc:pagename-display     optname-show-gains))
        (basis-method (get-option gnc:pagename-general     optname-basis-method))
        (prefer-pricelist (get-option gnc:pagename-general optname-prefer-pricelist))

        (total-basis    (gnc:make-commodity-collector))
        (total-value    (gnc:make-commodity-collector))
        (total-benefits (gnc:make-commodity-collector))
        (total-moneyin  (gnc:make-commodity-collector))
        (total-moneyout (gnc:make-commodity-collector))
        (total-gain     (gnc:make-commodity-collector)) ;; realized gain
        (total-ugain (gnc:make-commodity-collector))    ;; unrealized gain
        (total-allgain     (gnc:make-commodity-collector))
        ;(total-transfers '())
        (total-return (gnc:make-value-collector))
;;document will be the HTML document that we return.
        (table (gnc:make-html-table))
        (document (gnc:make-html-document)))

        (gnc:html-document-set-title!
          document (string-append 
                      report-title
                      (sprintf #f " %s" (gnc:print-date to-date))
                    ))
        (if (not (null? accounts))
; at least 1 account selected
          (let*((exchange-fn  (gnc:case-exchange-fn price-source currency to-date))
; when the change rate from foreign to domestic is unknown,
; search for the change rate from domestic to foreign
; before uncomment rename exchange-fn here above to exchange-fn-orig
                ;(exchange-fn
                ;  (lambda(foreign domestic)
                ;    (let
                ;      ((p (exchange-fn-orig foreign domestic)))
                ;      (if
                ;        (gnc:numeric-zero-p (gnc:gnc-monetary-amount p))
                ;        (let*
                ;          ( (d (gnc:make-gnc-monetary  domestic (gnc:make-gnc-numeric 10000 1) ))
                ;            (e (exchange-fn-orig d (gnc:gnc-monetary-commodity foreign))))
                ;          (set! p  (gnc:make-gnc-monetary
                ;                    domestic
                ;                    (gnc:numeric-mul(gnc:numeric-div
                ;                      (gnc:gnc-monetary-amount foreign)
                ;                      (gnc:gnc-monetary-amount e)
                ;                      1000000 
                ;                      GNC-RND-ROUND
                ;                    )(gnc:make-gnc-numeric 10000 1) 100
                ;                      GNC-RND-ROUND
                ;                  )))
                ;          (gnc:debug "exchange-fn inversed:"
                ;            (gnc:monetary->string d)
                ;            (gnc:monetary->string e)
                ;            (gnc:monetary->string p)
                ;           )
                ;        p)
                ;        (begin (gnc:debug "exchange-fn non-inversed:" (gnc:monetary->string p))
                ;      p)
                ;      )
                ;  )))
                (pricedb (gnc:book-get-pricedb (gnc:get-current-book)))
                (price-fn
                  (case price-source
                    ((pricedb-latest) 
                     (lambda (foreign date)
                        (gnc:pricedb-lookup-latest-any-currency pricedb foreign)
                     ))
                    ((pricedb-nearest) 
                     (lambda (foreign date)
                        (gnc:pricedb-lookup-nearest-in-time-any-currency
                          pricedb foreign (gnc:timepair-canonical-day-time date))
                     ))
                    ((pricedb-latest-before)
                     (lambda (foreign date)
                      (gnc:pricedb-lookup-latest-before-any-currency
                        pricedb foreign
                        (gnc:timepair-canonical-day-time date)
                      )))
                  ))
                (headercols (list (_ "Account")))
                (totalscols (list (gnc:make-html-table-cell/markup "total-label-cell" (_ "Total"))))
                (sum-total-benefits (gnc:numeric-zero))
                (sum-total-both-gains (gnc:numeric-zero))
                (sum-total-gain       (gnc:numeric-zero))
                (sum-total-ugain      (gnc:numeric-zero)))

;;begin building lists for which columns to display
              (if show-symbol 
                (begin
                  (append! headercols (list (_ "Symbol")))
                  (append! totalscols (list " "))))

              (if show-listing 
                (begin
                  (append! headercols (list (_ "Listing")))
                  (append! totalscols (list " "))))

              (if show-shares 
                (begin
                  (append! headercols (list (_ "Shares")))
                  (append! totalscols (list " "))))

              (if show-price 
                (begin
                  (append! headercols (list (_ "Price")))
                  (append! totalscols (list " "))))

              (append! headercols
                (list
                     " "
                  (_ "Basis")
                  (_ "Value")
                  (_ "Buy")
                  (_ "Sell")
                  ))
              (if show-gains (append! headercols (list (_ "Profit"))))
              (if (and show-gains include-gains) (append! headercols (list (_ "Income"))))
              (if show-gains (append! headercols (list (_ "Unrealized Gain"))))
              (append! headercols
                (list
                  (_ "Total Gain")
                  (_ "Total Return")
                ))
              (if show-return
                (append! headercols
                  (list    
                    (_ "Annual return")
                  )))
              

              (append! totalscols (list " "))

              (gnc:html-table-set-col-headers! table headercols)
          
              (table-add-stock-rows
                table accounts to-date currency price-fn exchange-fn
                include-empty include-gains show-symbol show-listing
                show-shares show-price show-return show-gains 
                basis-method prefer-pricelist
                total-basis total-value total-moneyin total-moneyout total-return
                total-benefits total-gain total-ugain total-allgain )
              
              (set! sum-total-gain (gnc:sum-collector-commodity total-gain currency exchange-fn))
              (set! sum-total-benefits (gnc:sum-collector-commodity total-benefits currency exchange-fn))
              (set! sum-total-ugain (gnc:sum-collector-commodity total-ugain currency exchange-fn))
              (set! sum-total-both-gains (gnc:sum-collector-commodity total-allgain currency exchange-fn))
              (set! sum-total-gain (gnc:sum-collector-commodity total-gain currency exchange-fn))
              
              ;(gnc:debug "total return rate" (total-return 'total #f))
              (gnc:html-table-append-row/markup!
                table
                "grand-total"
                (list
                  (gnc:make-html-table-cell/size
                    1
                    14
                    (gnc:make-html-text (gnc:html-markup-hr))
                  )))

;; finish building the totals columns, now that totals are complete
              (append!
               totalscols
               (list
                (gnc:make-html-table-cell/markup
                  "total-number-cell"
                  (gnc:sum-collector-commodity total-basis currency exchange-fn))
                (gnc:make-html-table-cell/markup
                  "total-number-cell"
                  (gnc:sum-collector-commodity total-value currency exchange-fn))
                (gnc:make-html-table-cell/markup
                  "total-number-cell"
                  (gnc:monetary-neg (gnc:sum-collector-commodity total-moneyin currency exchange-fn)))
                (gnc:make-html-table-cell/markup
                  "total-number-cell"
                  (gnc:sum-collector-commodity total-moneyout currency exchange-fn))))
              (if  show-gains
                (append!
                  totalscols
                  (list
                    (gnc:make-html-table-cell/markup
                      "total-number-cell"
                      sum-total-benefits)
                  )))
              (if
                (and show-gains  include-gains)
                (append!
                  totalscols
                  (list
                    (gnc:make-html-table-cell/markup
                      "total-number-cell"
                      sum-total-gain)
                  )))
            (if  show-gains
              (append!
               totalscols
               (list
                (gnc:make-html-table-cell/markup
                  "total-number-cell"
                  sum-total-ugain))))
            (append!
               totalscols
               (list
                  (gnc:make-html-table-cell/markup
                  "total-number-cell" sum-total-both-gains)
                (gnc:make-html-table-cell/markup
                  "total-number-cell" 
                  (let ((totalinvalue (gnc:numeric-to-double
                                        (gnc:gnc-monetary-amount (gnc:monetary-neg
                                                                    (gnc:sum-collector-commodity
                                                                      total-moneyin
                                                                      currency
                                                                      exchange-fn)
                                                                  )
                                        ))
                        ))
                      (if (= 0.0 totalinvalue)
                        (sprintf #f "%.2f%%" totalinvalue)
                        (sprintf #f "%.2f%%"
                            (* 100 (/ (gnc:numeric-to-double(gnc:gnc-monetary-amount sum-total-both-gains))
                                        totalinvalue)
                            )
                        )
                      )
                  ))))
              (if show-return
                (append!
                  totalscols
                  (list
                    (gnc:make-html-table-cell/markup
                      "total-number-cell" (sprintf #f "%.2f%%"(* (total-return 'total #f) 100))
                    )
                  )))
            
            (gnc:html-table-append-row/markup! table "grand-total" totalscols )
            (gnc:html-document-add-object! document table)
            (if warn-price-dirty 
              (gnc:html-document-append-objects!
                document 
                (list (gnc:make-html-text
                        (_ "* this commodity data was built using transaction pricing instead of the price list."))
                      (gnc:make-html-text (gnc:html-markup-br))
                      (gnc:make-html-text (_ "If you are in a multi-currency situation, the exchanges may not be correct."))
                )
              )
            )
          );; (let*((exchange-fn  
;if no accounts selected.
          (gnc:html-document-add-object!
            document
            (gnc:html-make-no-account-warning
              report-title (gnc:report-id report-obj))
          )
        )
    
        (gnc:report-finished)
    document
    );; let..
  )
); (define (advanced-portfolio-renderer report-obj)..

(gnc:define-report
 'version 1
 'name reportname
 'menu-path (list gnc:menuname-asset-liability)
 'options-generator options-generator
 'renderer advanced-portfolio-renderer)
